@@ -1,9 +1,9 @@
 
-#' @title Perform Wald test for a GMVAR or SGMVAR model
+#' @title Perform Wald test for a GMVAR, StMVAR, or G-StMVAR model
 #'
-#' @description \code{Wald_test} performs a Wald test for a GMVAR or SGMVAR model
+#' @description \code{Wald_test} performs a Wald test for a GMVAR, StMVAR, or G-StMVAR model
 #'
-#' @inheritParams simulateGMVAR
+#' @inheritParams quantile_residual_tests
 #' @inheritParams calc_gradient
 #' @param A a size \eqn{(k x n_params)} matrix with full row rank specifying part of the null hypothesis
 #'   where \eqn{n_params} is the number of parameters in the (unconstrained) model.
@@ -12,8 +12,8 @@
 #' @details Denoting the true parameter value by \eqn{\theta_{0}}, we test the null hypothesis \eqn{A\theta_{0}=c}.
 #'   Under the null, the test statistic is asymptotically \eqn{\chi^2}-distributed with \eqn{k}
 #'   (\code{=nrow(A)}) degrees of freedom. The parameter \eqn{\theta_{0}} is assumed to have the same form as in
-#'   the model supplied in the argument \code{gmvar} and it is presented in the documentation of the argument
-#'   \code{params} in the function \code{GMVAR} (see \code{?GMVAR}).
+#'   the model supplied in the argument \code{gsmvar} and it is presented in the documentation of the argument
+#'   \code{params} in the function \code{GSMVAR} (see \code{?GSMVAR}).
 #'
 #'   Finally, note that this function does \strong{not} check whether the specified constraints are feasible (e.g. whether
 #'   the implied constrained model would be stationary or have positive definite error term covariance matrices).
@@ -24,18 +24,18 @@
 #'   \item{alternative}{a character string describing the alternative hypothesis.}
 #'   \item{method}{a character string indicating the type of the test (Wald test).}
 #'   \item{data.name}{a character string giving the names of the supplied model, constraint matrix A, and vector c.}
-#'   \item{gmvar}{the supplied argument gmvar.}
+#'   \item{gsmvar}{the supplied argument gsmvar.}
 #'   \item{A}{the supplied argument A.}
 #'   \item{c}{the supplied argument c.}
 #'   \item{h}{the supplied argument h.}
-#' @seealso \code{\link{LR_test}}, \code{\link{fitGMVAR}}, \code{\link{GMVAR}}, \code{\link{diagnostic_plot}},
+#' @seealso \code{\link{LR_test}}, \code{\link{fitGSMVAR}}, \code{\link{GSMVAR}}, \code{\link{diagnostic_plot}},
 #'  \code{\link{profile_logliks}}, \code{\link{quantile_residual_tests}}, \code{\link{cond_moment_plot}}
 #' @inherit in_paramspace_int references
 #' @examples
 #' \donttest{
 #'  # Structural GMVAR(2, 2), d=2 model with recursive identification
 #'  W22 <- matrix(c(1, NA, 0, 1), nrow=2, byrow=FALSE)
-#'  fit22s <- fitGMVAR(gdpdef, p=2, M=2, structural_pars=list(W=W22),
+#'  fit22s <- fitGSMVAR(gdpdef, p=2, M=2, structural_pars=list(W=W22),
 #'                     ncalls=1, seeds=2)
 #'  fit22s
 #'
@@ -58,26 +58,27 @@
 #' }
 #' @export
 
-Wald_test <- function(gmvar, A, c, h=6e-6) {
-  params <- gmvar$params
+Wald_test <- function(gsmvar, A, c, h=6e-6) {
+  gsmvar <- gmvar_to_gsmvar(gsmvar) # Backward compatibility
+  params <- gsmvar$params
   stopifnot(is.matrix(A) && ncol(A) == length(params) && nrow(A) <= ncol(A))
   stopifnot(length(c) == nrow(A))
-  stopifnot(!is.null(gmvar$data))
+  stopifnot(!is.null(gsmvar$data))
   if(qr(A)$rank != nrow(A)) stop("The constraint matrix 'A' should have full row rank")
 
   # Calculate Hessian matrix at the estimate
-  minval <- get_minval(gmvar$data)
+  minval <- get_minval(gsmvar$data)
   loglik_fn <- function(pars) {
-    tryCatch(loglikelihood_int(data=gmvar$data, p=gmvar$model$p, M=gmvar$model$M, params=pars,
-                               conditional=gmvar$model$conditional,
-                               parametrization=gmvar$model$parametrization,
-                               constraints=gmvar$model$constraints,
-                               same_means=gmvar$model$same_means,
-                               structural_pars=gmvar$model$structural_pars,
+    tryCatch(loglikelihood_int(data=gsmvar$data, p=gsmvar$model$p, M=gsmvar$model$M, params=pars,
+                               conditional=gsmvar$model$conditional,
+                               parametrization=gsmvar$model$parametrization,
+                               constraints=gsmvar$model$constraints,
+                               same_means=gsmvar$model$same_means,
+                               structural_pars=gsmvar$model$structural_pars,
                                check_params=TRUE,
                                to_return="loglik", minval=minval,
-                               stat_tol=gmvar$num_tols$stat_tol,
-                               posdef_tol=gmvar$num_tols$posdef_tol),
+                               stat_tol=gsmvar$num_tols$stat_tol,
+                               posdef_tol=gsmvar$num_tols$posdef_tol),
              error=function(e) {
                print(paste("Failed to evualuate log-likelihood function in the approximation of Hessian matrix:", e))
                return(NA)
@@ -101,14 +102,14 @@ Wald_test <- function(gmvar, A, c, h=6e-6) {
   p_value <- pchisq(test_stat, df=df, lower.tail=FALSE)
 
   # Return
-  dname <- paste0(deparse(substitute(gmvar)),", ", deparse(substitute(A)), ", ", deparse(substitute(c)))
+  dname <- paste0(deparse(substitute(gsmvar)),", ", deparse(substitute(A)), ", ", deparse(substitute(c)))
   structure(list(statistic=c("W"=test_stat),
                  parameter=c("df"=df),
                  p.value=p_value,
                  alternative="the true parameter theta does not satisfy A%*%theta = c",
                  data.name=dname,
                  method="Wald test",
-                 gmvar=gmvar,
+                 gsmvar=gsmvar,
                  A=A,
                  c=c,
                  h=h),
@@ -116,13 +117,13 @@ Wald_test <- function(gmvar, A, c, h=6e-6) {
 }
 
 
-#' @title Perform likelihood ratio test for a GMVAR or SGMVAR model
+#' @title Perform likelihood ratio test for a GMVAR, StMVAR, or G-StMVAR model
 #'
-#' @description \code{LR_test} performs a likelihood ratio test for a GMVAR or SGMVAR model
+#' @description \code{LR_test} performs a likelihood ratio test for a GMVAR, StMVAR, or G-StMVAR model
 #'
-#' @param gmvar1 an object of class \code{'gmvar'} generated by \code{fitGMVAR} or \code{GMVAR}, containing
+#' @param gsmvar1 an object of class \code{'gsmvar'} generated by \code{fitGSMVAR} or \code{GSMVAR}, containing
 #'   the \strong{freely estimated} model.
-#' @param gmvar2 an object of class \code{'gmvar'} generated by \code{fitGMVAR} or \code{GMVAR}, containing
+#' @param gsmvar2 an object of class \code{'gsmvar'} generated by \code{fitGSMVAR} or \code{GSMVAR}, containing
 #'   the \strong{constrained} model.
 #' @details Performs a likelihood ratio test, testing the null hypothesis that the true parameter value lies
 #'   in the constrained parameter space. Under the null, the test statistic is asymptotically
@@ -137,9 +138,9 @@ Wald_test <- function(gmvar, A, c, h=6e-6) {
 #'   \item{alternative}{a character string describing the alternative hypothesis.}
 #'   \item{method}{a character string indicating the type of the test (likelihood ratio test).}
 #'   \item{data.name}{a character string giving the names of the supplied models, gsmar1 and gsmar2.}
-#'   \item{gmvar1}{the supplied argument gmvar1}
-#'   \item{gmvar2}{the supplied argument gmvar2}
-#' @seealso \code{\link{Wald_test}}, \code{\link{fitGMVAR}}, \code{\link{GMVAR}}, \code{\link{diagnostic_plot}},
+#'   \item{gsmvar1}{the supplied argument gsmvar1}
+#'   \item{gsmvar2}{the supplied argument gsmvar2}
+#' @seealso \code{\link{Wald_test}}, \code{\link{fitGSMVAR}}, \code{\link{GSMVAR}}, \code{\link{diagnostic_plot}},
 #'  \code{\link{profile_logliks}}, \code{\link{quantile_residual_tests}}, \code{\link{cond_moment_plot}}
 #' @inherit in_paramspace_int references
 #' @examples
@@ -149,13 +150,13 @@ Wald_test <- function(gmvar, A, c, h=6e-6) {
 #'
 #'  # Structural GMVAR(2, 2), d=2 model with recursive identification
 #'  W22 <- matrix(c(1, NA, 0, 1), nrow=2, byrow=FALSE)
-#'  fit22s <- fitGMVAR(gdpdef, p=2, M=2, structural_pars=list(W=W22),
+#'  fit22s <- fitGSMVAR(gdpdef, p=2, M=2, structural_pars=list(W=W22),
 #'                     ncalls=1, seeds=2)
 #'
 #'  # The same model but the AR coefficients restricted to be the same
 #'  # in both regimes:
 #'  C_mat <- rbind(diag(2*2^2), diag(2*2^2))
-#'  fit22sc <- fitGMVAR(gdpdef, p=2, M=2, constraints=C_mat,
+#'  fit22sc <- fitGSMVAR(gdpdef, p=2, M=2, constraints=C_mat,
 #'                      structural_pars=list(W=W22), ncalls=1, seeds=1)
 #'
 #'  # Test the AR constraints with likelihood ratio test:
@@ -163,26 +164,29 @@ Wald_test <- function(gmvar, A, c, h=6e-6) {
 #'  }
 #' @export
 
-LR_test <- function(gmvar1, gmvar2) {
-  check_gmvar(gmvar1, object_name="gmvar1")
-  check_gmvar(gmvar2, object_name="gmvar2")
-  stopifnot(length(gmvar1$params) > length(gmvar2$params))
-  stopifnot(gmvar1$loglik >= gmvar2$loglik)
+LR_test <- function(gsmvar1, gsmvar2) {
+  gsmvar1 <- gmvar_to_gsmvar(gsmvar1) # Backward compatibility
+  gsmvar2 <- gmvar_to_gsmvar(gsmvar2) # Backward compatibility
 
-  test_stat <- as.numeric(2*(gmvar1$loglik - gmvar2$loglik))
-  df <- length(gmvar1$params) - length(gmvar2$params)
+  check_gsmvar(gsmvar1, object_name="gsmvar1")
+  check_gsmvar(gsmvar2, object_name="gsmvar2")
+  stopifnot(length(gsmvar1$params) > length(gsmvar2$params))
+  stopifnot(gsmvar1$loglik >= gsmvar2$loglik)
+
+  test_stat <- as.numeric(2*(gsmvar1$loglik - gsmvar2$loglik))
+  df <- length(gsmvar1$params) - length(gsmvar2$params)
   p_value <- pchisq(test_stat, df=df, lower.tail=FALSE)
 
   # Return
-  dname <- paste(deparse(substitute(gmvar1)), "and", deparse(substitute(gmvar2)))
+  dname <- paste(deparse(substitute(gsmvar1)), "and", deparse(substitute(gsmvar2)))
   structure(list(statistic=c("LR"=test_stat),
                  parameter=c("df"=df),
                  p.value=p_value,
-                 alternative=paste("the true parameter does not satisfy the constraints imposed in", deparse(substitute(gmvar2))),
+                 alternative=paste("the true parameter does not satisfy the constraints imposed in", deparse(substitute(gsmvar2))),
                  data.name=dname,
                  method="Likelihood ratio test",
-                 gmvar1=gmvar1,
-                 gmvar2=gmvar2),
+                 gsmvar1=gsmvar1,
+                 gsmvar2=gsmvar2),
             class="htest")
 }
 
