@@ -11,14 +11,16 @@
 #' @inherit is_stationary references
 #' @keywords internal
 
-get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept", "mean"),
+get_regime_means_int <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), parametrization=c("intercept", "mean"),
                                  constraints=NULL, same_means=NULL, structural_pars=NULL) {
   parametrization <- match.arg(parametrization)
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints,
+  model <- match.arg(model)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, model=model, constraints=constraints,
                                     same_means=same_means, structural_pars=structural_pars)
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   if(parametrization == "intercept") {
-    params <- change_parametrization(p=p, M=M, d=d, params=params, constraints=NULL, structural_pars=structural_pars, change_to="mean")
+    params <- change_parametrization(p=p, M=M, d=d, params=params, model=model, constraints=NULL,
+                                     structural_pars=structural_pars, change_to="mean")
   }
   pick_phi0(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
 }
@@ -27,9 +29,9 @@ get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept",
 #' @title Calculate regime means \eqn{\mu_{m}}
 #'
 #' @description \code{get_regime_means} calculates regime means \eqn{\mu_{m} = (I - \sum A_{m,i})^(-1))}
-#'   for the given GMVAR model.
+#'   for the given GMVAR, StMVAR, or G-StMVAR model.
 #'
-#' @inheritParams simulateGMVAR
+#' @inheritParams quantile_residual_tests
 #' @return Returns a \eqn{(dxM)} matrix containing regime mean \eqn{\mu_{m}} in the m:th column, \eqn{m=1,..,M}.
 #' @family moment functions
 #' @seealso \code{\link{uncond_moments}}, \code{\link{get_regime_autocovs}}, \code{\link{cond_moments}}
@@ -39,7 +41,7 @@ get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept",
 #' params12 <- c(0.55, 0.112, 0.344, 0.055, -0.009, 0.718, 0.319, 0.005,
 #'  0.03, 0.619, 0.173, 0.255, 0.017, -0.136, 0.858, 1.185, -0.012,
 #'  0.136, 0.674)
-#' mod12 <- GMVAR(gdpdef, p=1, M=2, params=params12)
+#' mod12 <- GSMVAR(gdpdef, p=1, M=2, params=params12)
 #' mod12
 #' get_regime_means(mod12)
 #'
@@ -48,16 +50,18 @@ get_regime_means_int <- function(p, M, d, params, parametrization=c("intercept",
 #'  0.406, -0.005, 0.083, 0.299, 0.218, 0.02, -0.119, 0.722, 0.093, 0.032,
 #'  0.044, 0.191, 0.057, 0.172, -0.46, 0.016, 3.518, 5.154, 0.58)
 #' W_22 <- matrix(c(1, 1, -1, 1), nrow=2, byrow=FALSE)
-#' mod22s <- GMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
+#' mod22s <- GSMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
 #' mod22s
 #' get_regime_means(mod22s)
 #' @export
 
-get_regime_means <- function(gmvar) {
-  check_gmvar(gmvar)
-  get_regime_means_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
-                       parametrization=gmvar$model$parametrization, constraints=gmvar$model$constraints,
-                       same_means=gmvar$model$same_means, structural_pars=gmvar$model$structural_pars)
+get_regime_means <- function(gsmvar) {
+  gsmvar <- gmvar_to_gsmvar(gsmvar) # Backward compatibility
+  check_gsmvar(gsmvar)
+  get_regime_means_int(p=gsmvar$model$p, M=gsmvar$model$M, d=gsmvar$model$d, params=gsmvar$params,
+                       model=gsmvar$model$model, parametrization=gsmvar$model$parametrization,
+                       constraints=gsmvar$model$constraints, same_means=gsmvar$model$same_means,
+                       structural_pars=gsmvar$model$structural_pars)
 }
 
 
@@ -65,7 +69,7 @@ get_regime_means <- function(gmvar) {
 #' @title Calculate regimewise autocovariance matrices
 #'
 #' @description \code{get_regime_autocovs_int} calculates the regimewise autocovariance matrices \eqn{\Gamma_{m}(j)}
-#'  \eqn{j=0,1,...,p} for the given GMVAR model.
+#'  \eqn{j=0,1,...,p} for the given GSMVAR model.
 #'
 #' @inheritParams loglikelihood_int
 #' @inheritParams reform_constrained_pars
@@ -74,14 +78,15 @@ get_regime_means <- function(gmvar) {
 #' @inherit loglikelihood_int references
 #' @keywords internal
 
-get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, same_means=NULL, structural_pars=NULL) {
-
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints,
+get_regime_autocovs_int <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), constraints=NULL, same_means=NULL, structural_pars=NULL) {
+  model <- match.arg(model)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, model=model, constraints=constraints,
                                     same_means=same_means, structural_pars=structural_pars)
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
   all_A <- pick_allA(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
   all_Omega <- pick_Omegas(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
   all_boldA <- form_boldA(p=p, M=M, d=d, all_A=all_A)
+  M <- sum(M)
   I_dp2 <- diag(nrow=(d*p)^2)
   ZER_lower <- matrix(0, nrow=d*(p-1), ncol=d*p)
   ZER_right <- matrix(0, nrow=d, ncol=d*(p - 1))
@@ -105,9 +110,9 @@ get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, same_mean
 #' @title Calculate regimewise autocovariance matrices
 #'
 #' @description \code{get_regime_autocovs} calculates the first p regimewise autocovariance
-#'  matrices \eqn{\Gamma_{m}(j)} for the given GMVAR model.
+#'  matrices \eqn{\Gamma_{m}(j)} for the given GMVAR, StMVAR, or G-StMVAR model.
 #'
-#' @inheritParams simulateGMVAR
+#' @inheritParams quantile_residual_tests
 #' @family moment functions
 #' @inherit get_regime_autocovs_int return
 #' @inherit loglikelihood_int references
@@ -116,7 +121,7 @@ get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, same_mean
 #' params12 <- c(0.55, 0.112, 0.344, 0.055, -0.009, 0.718, 0.319, 0.005,
 #'  0.03, 0.619, 0.173, 0.255, 0.017, -0.136, 0.858, 1.185, -0.012,
 #'  0.136, 0.674)
-#' mod12 <- GMVAR(gdpdef, p=1, M=2, params=params12)
+#' mod12 <- GSMVAR(gdpdef, p=1, M=2, params=params12)
 #' get_regime_autocovs(mod12)
 #'
 #' # Structural GMVAR(2, 2), d=2 model identified with sign-constraints:
@@ -124,24 +129,25 @@ get_regime_autocovs_int <- function(p, M, d, params, constraints=NULL, same_mean
 #'  0.406, -0.005, 0.083, 0.299, 0.218, 0.02, -0.119, 0.722, 0.093, 0.032,
 #'  0.044, 0.191, 0.057, 0.172, -0.46, 0.016, 3.518, 5.154, 0.58)
 #' W_22 <- matrix(c(1, 1, -1, 1), nrow=2, byrow=FALSE)
-#' mod22s <- GMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
+#' mod22s <- GSMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
 #' mod22s
 #' get_regime_autocovs(mod22s)
 #' @export
 
-get_regime_autocovs <- function(gmvar) {
-  check_gmvar(gmvar)
-  get_regime_autocovs_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
-                          constraints=gmvar$model$constraints, same_means=gmvar$model$same_means,
-                          structural_pars=gmvar$model$structural_pars)
+get_regime_autocovs <- function(gsmvar) {
+  gsmvar <- gmvar_to_gsmvar(gsmvar) # Backward compatibility
+  check_gsmvar(gsmvar)
+  get_regime_autocovs_int(p=gsmvar$model$p, M=gsmvar$model$M, d=gsmvar$model$d, params=gsmvar$params,
+                          model=gsmvar$model$model, constraints=gsmvar$model$constraints,
+                          same_means=gsmvar$model$same_means, structural_pars=gsmvar$model$structural_pars)
 }
 
 
 #' @title Calculate the unconditional mean, variance, the first p autocovariances, and the first p autocorrelations
-#'  of a GMVAR process
+#'  of a GMVAR, StMVAR, or G-StMVAR process
 #'
 #' @description \code{uncond_moments_int} calculates the unconditional mean, variance, the first p autocovariances,
-#'  and the first p autocorrelations of the specified GMVAR process.
+#'  and the first p autocorrelations of the specified GMVAR, StMVAR, or G-StMVAR process.
 #'
 #' @inheritParams loglikelihood_int
 #' @inheritParams reform_constrained_pars
@@ -156,19 +162,20 @@ get_regime_autocovs <- function(gmvar) {
 #' @inherit loglikelihood_int references
 #' @keywords internal
 
-uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "mean"), constraints=NULL,
-                               same_means=NULL, structural_pars=NULL) {
+uncond_moments_int <- function(p, M, d, params, model=c("GMVAR", "StMVAR", "G-StMVAR"), parametrization=c("intercept", "mean"),
+                               constraints=NULL, same_means=NULL, structural_pars=NULL) {
   parametrization <- match.arg(parametrization)
-  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, constraints=constraints, same_means=same_means,
+  model <- match.arg(model)
+  params <- reform_constrained_pars(p=p, M=M, d=d, params=params, model=model, constraints=constraints, same_means=same_means,
                                     structural_pars=structural_pars) # Remove any constraints
   structural_pars <- get_unconstrained_structural_pars(structural_pars=structural_pars)
-  alphas <- pick_alphas(p=p, M=M, d=d, params=params)
-  reg_means <- get_regime_means_int(p=p, M=M, d=d, params=params, parametrization=parametrization, constraints=NULL,
-                                    same_means=NULL, structural_pars=structural_pars)
+  alphas <- pick_alphas(p=p, M=M, d=d, params=params, model=model)
+  reg_means <- get_regime_means_int(p=p, M=M, d=d, params=params, model=model, parametrization=parametrization,
+                                    constraints=NULL, same_means=NULL, structural_pars=structural_pars)
   uncond_mean <- colSums(alphas*t(reg_means))
-  tmp <- rowSums(vapply(1:M, function(m) alphas[m]*tcrossprod(reg_means[,m] - uncond_mean), numeric(d*d))) # Vectorized matrix
-  reg_autocovs <- get_regime_autocovs_int(p=p, M=M, d=d, params=params, constraints=NULL, structural_pars=structural_pars)
-  autocovs <- array(rowSums(vapply(1:M, function(m) alphas[m]*reg_autocovs[, , , m], numeric(d*d*(p + 1)))) + tmp, dim=c(d, d, p + 1))
+  tmp <- rowSums(vapply(1:sum(M), function(m) alphas[m]*tcrossprod(reg_means[,m] - uncond_mean), numeric(d*d))) # Vectorized matrix
+  reg_autocovs <- get_regime_autocovs_int(p=p, M=M, d=d, params=params, model=model, constraints=NULL, structural_pars=structural_pars)
+  autocovs <- array(rowSums(vapply(1:sum(M), function(m) alphas[m]*reg_autocovs[, , , m], numeric(d*d*(p + 1)))) + tmp, dim=c(d, d, p + 1))
   ind_vars <- diag(autocovs[, , 1])
   autocors <- array(vapply(1:(p + 1), function(i1) {
     acor_mat <- matrix(NA, nrow=d, ncol=d)
@@ -187,12 +194,12 @@ uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "
 
 
 #' @title Calculate the unconditional mean, variance, the first p autocovariances, and the first p autocorrelations
-#'  of a GMVAR process
+#'  of a GMVAR, StMVAR, or G-StMVAR process
 #'
 #' @description \code{uncond_moments} calculates the unconditional mean, variance, the first p autocovariances,
-#'  and the first p autocorrelations of the given GMVAR process.
+#'  and the first p autocorrelations of the given GMVAR, StMVAR, or G-StMVAR process.
 #'
-#' @inheritParams simulateGMVAR
+#' @inheritParams quantile_residual_tests
 #' @family moment functions
 #' @inherit uncond_moments_int return
 #' @inherit uncond_moments_int details references
@@ -201,7 +208,7 @@ uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "
 #' params12 <- c(0.55, 0.112, 0.344, 0.055, -0.009, 0.718, 0.319, 0.005,
 #'  0.03, 0.619, 0.173, 0.255, 0.017, -0.136, 0.858, 1.185, -0.012,
 #'  0.136, 0.674)
-#' mod12 <- GMVAR(gdpdef, p=1, M=2, params=params12)
+#' mod12 <- GSMVAR(gdpdef, p=1, M=2, params=params12)
 #' uncond_moments(mod12)
 #'
 #' # Structural GMVAR(2, 2), d=2 model identified with sign-constraints:
@@ -209,16 +216,18 @@ uncond_moments_int <- function(p, M, d, params, parametrization=c("intercept", "
 #'  0.406, -0.005, 0.083, 0.299, 0.218, 0.02, -0.119, 0.722, 0.093, 0.032,
 #'  0.044, 0.191, 0.057, 0.172, -0.46, 0.016, 3.518, 5.154, 0.58)
 #' W_22 <- matrix(c(1, 1, -1, 1), nrow=2, byrow=FALSE)
-#' mod22s <- GMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
+#' mod22s <- GSMVAR(gdpdef, p=2, M=2, params=params22s, structural_pars=list(W=W_22))
 #' mod22s
 #' uncond_moments(mod22s)
 #' @export
 
-uncond_moments <- function(gmvar) {
-  check_gmvar(gmvar)
-  uncond_moments_int(p=gmvar$model$p, M=gmvar$model$M, d=gmvar$model$d, params=gmvar$params,
-                     parametrization=gmvar$model$parametrization, constraints=gmvar$model$constraints,
-                     same_means=gmvar$model$same_means, structural_pars=gmvar$model$structural_pars)
+uncond_moments <- function(gsmvar) {
+  gsmvar <- gmvar_to_gsmvar(gsmvar) # Backward compatibility
+  check_gsmvar(gsmvar)
+  uncond_moments_int(p=gsmvar$model$p, M=gsmvar$model$M, d=gsmvar$model$d, params=gsmvar$params,
+                     model=gsmvar$model$model, parametrization=gsmvar$model$parametrization,
+                     constraints=gsmvar$model$constraints, same_means=gsmvar$model$same_means,
+                     structural_pars=gsmvar$model$structural_pars)
 }
 
 
@@ -328,10 +337,10 @@ VAR_pcovmat <- function(p, d, all_Am, Omega_m) {
 
 
 #' @title Calculate the dp-dimensional covariance matrices \eqn{\Sigma_{m,p}} in the mixing weights
-#'  of the GMVAR model.
+#'  of the GMVAR, StMVAR, or G-StMVAR model.
 #'
 #' @description \code{get_Sigmas} calculates the dp-dimensional covariance matrices \eqn{\Sigma_{m,p}}
-#'  in the mixing weights of the GMVAR model so that the algorithm proposed by McElroy (2017) employed
+#'  in the mixing weights of the GMVAR, StMVAR, or G-StMVAR model so that the algorithm proposed by McElroy (2017) employed
 #'  whenever it reduces the computation time.
 #'
 #' @inheritParams is_stationary
@@ -353,10 +362,13 @@ VAR_pcovmat <- function(p, d, all_Am, Omega_m) {
 #'            \emph{Springer}.
 #'    \item McElroy T. 2017. Computation of vector ARMA autocovariances.
 #'          \emph{Statistics and Probability Letters}, \strong{124}, 92-96.
+#'    \item Virolainen S. 2021. Gaussian and Student's t mixture vector autoregressive model. Unpublished working
+#'      paper, available as arXiv:2109.13648.
 #'  }
 #' @keywords internal
 
 get_Sigmas <- function(p, M, d, all_A, all_boldA, all_Omega) {
+  M <- sum(M)
   Sigmas <- array(NA, dim=c(d*p, d*p, M)) # Store the (dpxdp) covariance matrices
   if(d*p < 12) { # d*p < 12
     # Calculate the covariance matrices Sigma_{m,p} using the equation (2.1.39) in Lütkepohl (2005).
