@@ -1,7 +1,6 @@
 context("argument checking functions")
 library(gmvarkit)
 
-# The tests are very brief !!!
 
 ## A(M)(p)_(p)(M)(d)
 
@@ -18,6 +17,10 @@ Omega1_112s <- tcrossprod(W_112)
 
 theta_112sWC <- c(phi10_112, vec(A11_112), Wvec(W_112)) # SGMVAR W constrained
 
+theta_112t <- c(theta_112, 10)
+theta_112tsWC <- c(theta_112sWC, 10)
+
+
 # p=1, M=2, d=2
 phi10_122 <- c(1.03, 2.36)
 A11_122 <- matrix(c(0.9, 0.3, -0.3, 0.9), nrow=2, byrow=FALSE)
@@ -59,6 +62,10 @@ lambdas_222 <- WL_222[(2^2 + 1):length(WL_222)]
 theta_222s <- c(phi10_222, phi20_222, vec(A11_222), vec(A12_222), vec(A21_222),
                 vec(A22_222), vec(W_222), lambdas_222, alpha1_222) # SGMVAR
 
+theta_222gs <- c(theta_222, 20) # G-StMVAR, M1=1, M2=1
+theta_222ts <- c(theta_222s, 10, 20) # SStMVAR
+
+
 # p=3, M=3, d=2
 phi10_332 <- c(1.03, 2.36)
 A11_332 <- matrix(c(1.25, 0.06, 0.04, 1.34), nrow=2, byrow=FALSE)
@@ -103,6 +110,11 @@ theta_332sWC <- c(phi10_332, phi20_332, phi30_332, vec(A11_332), vec(A12_332), v
                   vec(A21_332), vec(A22_332), vec(A23_332), vec(A31_332), vec(A32_332), vec(A33_332),
                   Wvec(W_332), lambdas2_332, lambdas3_332, alpha1_332, alpha2_332) # SGMVAR W constrained
 
+
+theta_332gs <- c(theta_332, 20, 30) # G-StMVAR, M1=1, M2=2
+theta_332gssWC <- c(theta_332sWC, 30) # SG-StMVAR, M1=2, M2=1
+
+
 # p=1, M=2, d=3
 phi10_123 <- c(1.1, 2.2, 3.3)
 A11_123 <- matrix(c(0.1, 0.21, 0.31, 0.12, 0.2, 0.32, 0.13, 0.23, 0.1), nrow=3, byrow=FALSE)
@@ -122,6 +134,10 @@ W_123 <- matrix(WL_123[1:(3^2)], nrow=3, byrow=FALSE)
 lambdas_123 <- WL_123[(3^2 + 1):length(WL_123)]
 theta_123s <- c(phi10_123, phi20_123, vec(A11_123), vec(A21_123), vec(W_123), lambdas_123, alpha1_123) # SGMVAR
 
+theta_123t <- c(theta_123, 10, 20) # StMVAR
+theta_123gss <- c(theta_123s, 20) # SG-StMVAR
+
+
 # p=2, M=1, d=3
 phi10_213 <- c(1.1, 2.2, 3.3)
 A11_213 <- matrix(c(1, 0.21, 0.31, 0.12, 1.3, 0.32, 0.13, 0.23, 1), nrow=3, byrow=FALSE)
@@ -136,6 +152,9 @@ theta_213s <- c(phi10_213, vec(A11_213), vec(A12_213), vec(W_213)) # SGMVAR
 
 theta_213sWC <- c(phi10_213, vec(A11_213), vec(A12_213), Wvec(W_213)) # SGMVAR W constrained
 
+theta_213t <- c(theta_213, 10) # StMVAR
+theta_213tsWC <- c(theta_213, 10) # SStMVAR
+
 
 # Constraining AR-parameters to be the same for all regimes
 rbind_diags <- function(p, M, d) {
@@ -155,6 +174,11 @@ theta_122c_expanded <- c(phi10_122, vec(A11_122), vech(Omega1_122), phi20_122, v
 
 theta_112csWAR <- c(phi10_112, vec(A11_112), Wvec(W_112)) # SGMVAR W and AR
 
+theta_112tc <- c(theta_112c, 10) # StMVAR
+
+theta_112tcsWAR <- c(theta_112csWAR, 10) # StMVAR
+
+
 # p=1, M=2, d=2
 C_222 <- rbind_diags(p=2, M=2, d=2)
 theta_222c <- c(phi10_222, phi20_222, vec(A11_222), vec(A12_222), vech(Omega1_222), vech(Omega2_222), alpha1_222)
@@ -176,6 +200,12 @@ theta_222csLAR <- c(phi10_222, phi20_222, vec(A11_222), vec(A12_222), vec(W_222)
 theta_222csLAR_expanded <- c(phi10_222, phi20_222, vec(A11_222), vec(A12_222), vec(A11_222), vec(A12_222),
                              vec(W_222), 0.2, 2*0.2, alpha1_222) # SGMVAR lambdas and AR
 
+theta_222gscsL <- c(theta_222csL, 20) # SG-StMVAR, M1=1, M2=1
+theta_222gscsL_expanded <- c(theta_222csL_expanded, 20) # SG-StMVAR, M1=1, M2=1
+
+theta_222tcsLAR <- c(theta_222csLAR, 10, 20) # SStMVAR
+theta_222tcsLAR_expanded <- c(theta_222csLAR_expanded, 10, 20) # SStMVAR
+
 
 # p=3, M=3, d=2
 C_332 <- rbind_diags(p=3, M=3, d=2)
@@ -202,6 +232,13 @@ theta_332csWLAR_expanded <- c(phi10_332, phi20_332, phi30_332, vec(A11_332), vec
                               vec(A11_332), vec(A12_332), vec(A13_332), vec(W_332), 1, 1, 2, 2, alpha1_332, alpha2_332)
 
 
+theta_332gsc <- c(theta_332c, 30) # G-StMVAR, M1=2, M2=1
+theta_332gsc_expanded <- c(theta_332c_expanded, 30) # G-StMVAR, M1=2, M2=1
+
+theta_332gscsWLAR <- c(theta_332csWLAR, 20, 30) # SG-StMVAR, M1=1, M2=2
+theta_332gscsWLAR_expanded <- c(theta_332csWLAR_expanded, 20, 30) # SG-StMVAR, M1=1, M2=2
+
+
 # p=1, M=2, d=3
 C_123 <- rbind_diags(p=1, M=2, d=3)
 theta_123c <- c(phi10_123, phi20_123, vec(A11_123), vech(Omega1_123), vech(Omega2_123), alpha1_123)
@@ -213,6 +250,13 @@ theta_123csL_expanded <- c(phi10_123, phi20_123, vec(A11_123), vec(A21_123), vec
 theta_123csLAR <- c(phi10_123, phi20_123, vec(A11_123), vec(W_123), 1, 2, alpha1_123) # SGMVAR lambdas and AR
 theta_123csLAR_expanded <- c(phi10_123, phi20_123, vec(A11_123), vec(A11_123), vec(W_123), 1, 1, 2, alpha1_123)
 
+theta_123tc <- c(theta_123c, 10, 20) # StMVAR
+theta_123tc_expanded <- c(theta_123c_expanded, 10, 20) # StMVAR
+
+theta_123tcsL <- c(theta_123csL, 10, 20) # StMVAR
+theta_123tcL_expanded <- c(theta_123csL_expanded, 10, 20) # StMVAR
+
+
 # p=2, M=1, d=3
 C_213 <- rbind_diags(p=2, M=1, d=3)
 theta_213c <- c(phi10_213, vec(A11_213), vec(A12_213), vech(Omega1_213))
@@ -221,6 +265,10 @@ C_213 <- rbind_diags(p=2, M=1, d=3)
 theta_213c <- c(phi10_213, vec(A11_213), vec(A12_213), vech(Omega1_213))
 theta_213csWAR <- c(phi10_213, vec(A11_213), vec(A12_213), Wvec(W_213))
 
+theta_213tc <- c(theta_213c, 10) # StMVAR
+
+theta_213tcsWAR <- c(theta_213csWAR, 10) # SStMVAR
+
 # p=2, M=2, d=2, constraint AR-parameters to be the same for all regimes
 # and constraint the of-diagonal elements of AR-matrices to be zero.
 mat0 <- matrix(c(1, rep(0, 10), 1, rep(0, 8), 1, rep(0, 10), 1), nrow=2*2^2, byrow=FALSE)
@@ -256,6 +304,9 @@ theta_112_int_expanded <- theta_112_int
 theta_122_int <- c(phi10_122, vec(A11_122), vec(A21_122), vech(Omega1_122), vech(Omega2_122), alpha1_122)
 theta_122_int_expanded <- c(phi10_122, vec(A11_122), vech(Omega1_122), phi10_122, vec(A21_122), vech(Omega2_122), alpha1_122)
 
+theta_122t_int <- c(theta_122_int, 10, 20) # StMVAR
+theta_122t_int_expanded <- c(theta_122_int_expanded, 10, 20) # StMVAR
+
 # p=1, M=2, d=2, same_means=list(1, 2)
 theta_122_int2 <- c(phi10_122, phi20_122, vec(A11_122), vec(A21_122), vech(Omega1_122), vech(Omega2_122), alpha1_122)
 theta_122_int2_expanded <- c(phi10_122, vec(A11_122), vech(Omega1_122), phi20_122, vec(A21_122), vech(Omega2_122), alpha1_122)
@@ -274,6 +325,10 @@ theta_222csLAR_int <- c(phi10_222, vec(A11_222), vec(A12_222), vec(W_222), 0.2,
 theta_222csLAR_int_expanded <-  c(phi10_222, phi10_222, vec(A11_222), vec(A12_222), vec(A11_222), vec(A12_222),
                                   vec(W_222), 0.2, 2*0.2, alpha1_222)
 
+theta_222tcsLAR_int <- c(theta_222csLAR_int, 10, 20) # StMVAR
+theta_222tcsLAR_int_expanded <- c(theta_222csLAR_int_expanded, 10, 20) # StMVAR
+
+
 # p=3, M=3, d=2, constraints=C_332, same_means=list(1, 2:3)
 theta_332c_int <- c(phi10_332, phi20_332, vec(A11_332), vec(A12_332), vec(A13_332), vech(Omega1_332), vech(Omega2_332),
                     vech(Omega3_332), alpha1_332, alpha2_332)
@@ -281,6 +336,10 @@ theta_332c_int_expanded <- c(phi10_332, vec(A11_332), vec(A12_332), vec(A13_332)
                              vec(A12_332), vec(A13_332), vech(Omega2_332), phi20_332, vec(A11_332), vec(A12_332), vec(A13_332),
                              vech(Omega3_332), alpha1_332, alpha2_332)
 
+theta_332gsc_int <- c(theta_332c_int, 30) # G-StMVAR, M1=2, M2=1
+theta_332gsc_int_expanded <- c(theta_332c_int_expanded, 30) # G-StMVAR, M1=2, M2=1
+
+
 # p=3, M=3, d=2, constraints=C_332, same_means=list(2, c(1, 3))
 theta_332c_int2 <- c(phi10_332, phi20_332, vec(A11_332), vec(A12_332), vec(A13_332), vech(Omega1_332), vech(Omega2_332),
                      vech(Omega3_332), alpha1_332, alpha2_332)
@@ -288,6 +347,9 @@ theta_332c_int2_expanded <- c(phi20_332, vec(A11_332), vec(A12_332), vec(A13_332
                               vec(A12_332), vec(A13_332), vech(Omega2_332), phi20_332, vec(A11_332), vec(A12_332), vec(A13_332),
                               vech(Omega3_332), alpha1_332, alpha2_332)
 
+theta_332gsc_int2 <- c(theta_332c_int2, 20, 30) # G-StMVAR, M1=1, M2=2
+theta_332gsc_int2_expanded <- c(theta_332c_int2_expanded, 20, 30) # G-StMVAR, M1=1, M2=2
+
 # p=3, M=3, d=2, constraints=C_332, same_means=list(1:3)
 theta_332c_int3 <- c(phi10_332, vec(A11_332), vec(A12_332), vec(A13_332), vech(Omega1_332), vech(Omega2_332),
                      vech(Omega3_332), alpha1_332, alpha2_332)
@@ -302,12 +364,20 @@ theta_332csWAR_int_expanded <- c(phi10_332, phi20_332, phi10_332, vec(A11_332),
                                  vec(A12_332), vec(A13_332), vec(A11_332), vec(A12_332), vec(A13_332), vec(W_332), lambdas2_332,
                                  lambdas3_332, alpha1_332, alpha2_332)
 
+theta_332tcsWAR_int <- c(theta_332csWAR_int, 10, 20, 30) # SStMVAR
+theta_332tcsWAR_int_expanded <- c(theta_332csWAR_int_expanded, 10, 20, 30) # SStMVAR
+
+
 # p=3, M=3, d=2, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(1:2, 3)
 theta_332csWL_int <- c(phi10_332, phi30_332, vec(A11_332), vec(A12_332), vec(A13_332), vec(A21_332), vec(A22_332), vec(A23_332),
                        vec(A31_332), vec(A32_332), vec(A33_332), Wvec(W_332), 1, 2, alpha1_332, alpha2_332)
 theta_332csWL_int_expanded <- c(phi10_332, phi10_332, phi30_332, vec(A11_332), vec(A12_332), vec(A13_332), vec(A21_332), vec(A22_332), vec(A23_332),
                                 vec(A31_332), vec(A32_332), vec(A33_332), vec(W_332), 1, 1, 2, 2, alpha1_332, alpha2_332)
 
+theta_332gscsWL_int <- c(theta_332csWL_int, 20, 30) # SG-StMVAR, M1=1, M2=2
+theta_332gscsWL_int_expanded <- c(theta_332csWL_int_expanded, 20, 30) # SG-StMVAR, M1=1, M2=2
+
+
 # p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(2:3, 1)
 theta_332csWLAR_int <- c(phi10_332, phi20_332, vec(A11_332), vec(A12_332), vec(A13_332), Wvec(W_332), 1, 2,
                          alpha1_332, alpha2_332)
@@ -318,6 +388,9 @@ theta_332csWLAR_int_expanded <- c(phi20_332, phi10_332, phi10_332, vec(A11_332),
 theta_123_int <- c(phi10_123, vec(A11_123), vec(A21_123), vech(Omega1_123), vech(Omega2_123), alpha1_123)
 theta_123_int_expanded <- c(phi10_123, vec(A11_123), vech(Omega1_123), phi10_123, vec(A21_123), vech(Omega2_123), alpha1_123)
 
+theta_123gs_int <- c(theta_123_int, 20) # G-StMVAR, M1=1, M2=1
+theta_123gs_int_expanded <- c(theta_123_int_expanded, 20) # G-StMVAR, M1=1, M2=1
+
 # p=1, M=2, d=3, constraints=C_123, same_means=list(1:2)
 theta_123c_int <- c(phi10_123, vec(A11_123), vech(Omega1_123), vech(Omega2_123), alpha1_123)
 theta_123c_int_expanded <- c(phi10_123, vec(A11_123), vech(Omega1_123), phi10_123, vec(A11_123), vech(Omega2_123), alpha1_123)
@@ -326,6 +399,10 @@ theta_123c_int_expanded <- c(phi10_123, vec(A11_123), vech(Omega1_123), phi10_12
 theta_123csL_int <- c(phi10_123, vec(A11_123), vec(A21_123), vec(W_123), 1, 2, alpha1_123)
 theta_123csL_int_expanded <- c(phi10_123, phi10_123, vec(A11_123), vec(A21_123), vec(W_123), 1, 1, 2, alpha1_123)
 
+theta_123tcsL_int <- c(theta_123csL_int, 10, 20) # SStMVAR
+theta_123tcsL_int_expanded <- c(theta_123csL_int_expanded, 10, 20) # SStMVAR
+
+
 # p=1, M=2, d=3, constraints=C_123, structural_pars=list(W=W_123, C_lambda=C_lambda_123) same_means=list(1:2)
 theta_123csLAR_int <- c(phi10_123, vec(A11_123), vec(W_123), 1, 2, alpha1_123) # SGMVAR lambdas and AR
 theta_123csLAR_int_expanded <- c(phi10_123, phi10_123, vec(A11_123), vec(A11_123), vec(W_123), 1, 1, 2, alpha1_123)
@@ -339,38 +416,68 @@ theta_213csWAR_int <- c(phi10_213, vec(A11_213), vec(A12_213), Wvec(W_213))
 theta_213csWAR_int_expanded <- c(phi10_213, vec(A11_213), vec(A12_213), vec(W_213))
 
 
+
+#############
+theta_122_ok <- c(0.55, 0.112, 0.344, 0.055, -0.009, 0.718, 0.319, 0.005, 0.03,
+                  0.619, 0.173, 0.255, 0.017, -0.136, 0.858, 1.185, -0.012, 0.136, 0.674)
+theta_122t_df <- c(theta_122_ok, 3, 2) # StMVAR, too small df
+theta_122gs_df <- c(theta_122_ok, 2) # G-StMVAR, M1=1, M2=1, too small df
+
+
+
 test_that("is_stationary works correctly", {
   expect_false(is_stationary(p=1, M=1, d=2, params=theta_112))
+  expect_false(is_stationary(p=1, M=1, d=2, params=theta_112t))
   expect_true(is_stationary(p=2, M=2, d=2, params=theta_222))
+  expect_true(is_stationary(p=2, M=c(1, 1), d=2, params=theta_222gs))
   expect_false(is_stationary(p=3, M=3, d=2, params=theta_332))
+  expect_false(is_stationary(p=3, M=c(1, 2), d=2, params=theta_332gs))
   expect_false(is_stationary(p=1, M=2, d=3, params=theta_123))
+  expect_false(is_stationary(p=1, M=2, d=3, params=theta_123t))
   expect_false(is_stationary(p=2, M=1, d=3, params=theta_213))
+  expect_false(is_stationary(p=2, M=1, d=3, params=theta_213t))
 
-  # SGMVAR
+  # Structural
   expect_false(is_stationary(p=1, M=1, d=2, params=theta_112sWC, structural_pars=list(W=W_112)))
+  expect_false(is_stationary(p=1, M=1, d=2, params=theta_112tsWC, structural_pars=list(W=W_112)))
   expect_true(is_stationary(p=2, M=2, d=2, params=theta_222s, structural_pars=list(W=W_222)))
+  expect_true(is_stationary(p=2, M=2, d=2, params=theta_222ts, structural_pars=list(W=W_222)))
   expect_false(is_stationary(p=3, M=3, d=2, params=theta_332sWC, structural_pars=list(W=W_332)))
+  expect_false(is_stationary(p=3, M=c(2, 1), d=2, params=theta_332gssWC, structural_pars=list(W=W_332)))
   expect_false(is_stationary(p=1, M=2, d=3, params=theta_123s, structural_pars=list(W=W_123)))
+  expect_false(is_stationary(p=1, M=c(1, 1), d=3, params=theta_123gss, structural_pars=list(W=W_123)))
   expect_false(is_stationary(p=2, M=1, d=3, params=theta_213sWC, structural_pars=list(W=W_213)))
+  expect_false(is_stationary(p=2, M=1, d=3, params=theta_213tsWC, structural_pars=list(W=W_213)))
 })
 
-test_that("in_paramspace works correctly", {
+test_that("in_paramspace works correctly", { # calls in_paramspace_int
   expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112))
+  expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112t, model="StMVAR"))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222))
+  expect_true(in_paramspace(p=2, M=c(1, 1), d=2, params=theta_222gs, model="G-StMVAR"))
   expect_false(in_paramspace(p=3, M=3, d=2, params=theta_332))
+  expect_false(in_paramspace(p=3, M=c(1, 2), d=2, params=theta_332gs, model="G-StMVAR"))
   expect_false(in_paramspace(p=1, M=2, d=3, params=theta_123))
+  expect_false(in_paramspace(p=1, M=2, d=3, params=theta_123t, model="StMVAR"))
   expect_false(in_paramspace(p=2, M=1, d=3, params=theta_213))
+  expect_false(in_paramspace(p=2, M=1, d=3, params=theta_213t, model="StMVAR"))
 
   expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112c, constraints=C_112))
+  expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112tc, model="StMVAR", constraints=C_112))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222c, constraints=C_222))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222_c2, constraints=C_222_2))
 
-  # SGMVAR
+  # Structural
   expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112sWC, structural_pars=list(W=W_112)))
+  expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112tsWC, model="StMVAR", structural_pars=list(W=W_112)))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222s, structural_pars=list(W=W_222)))
+  expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222ts, model="StMVAR", structural_pars=list(W=W_222)))
   expect_false(in_paramspace(p=3, M=3, d=2, params=theta_332sWC, structural_pars=list(W=W_332)))
+  expect_false(in_paramspace(p=3, M=c(2, 1), d=2, params=theta_332gssWC, model="G-StMVAR", structural_pars=list(W=W_332)))
   expect_false(in_paramspace(p=1, M=2, d=3, params=theta_123s, structural_pars=list(W=W_123)))
+  expect_false(in_paramspace(p=1, M=c(1, 1), d=3, params=theta_123gss, model="G-StMVAR", structural_pars=list(W=W_123)))
   expect_false(in_paramspace(p=2, M=1, d=3, params=theta_213sWC, structural_pars=list(W=W_213)))
+  expect_false(in_paramspace(p=2, M=1, d=3, params=theta_213tsWC, model="StMVAR", structural_pars=list(W=W_213)))
 
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222csLAR, constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222)))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222_c2s, constraints=C_222_2, structural_pars=list(W=W_222c2)))
@@ -389,30 +496,43 @@ test_that("in_paramspace works correctly", {
   # same_means
   expect_false(in_paramspace(p=1, M=1, d=2, params=theta_112_int, same_means=list(1)))
   expect_true(in_paramspace(p=1, M=2, d=2, params=theta_122_int, same_means=list(1:2)))
+  expect_true(in_paramspace(p=1, M=2, d=2, params=theta_122t_int, model="StMVAR", same_means=list(1:2)))
   expect_true(in_paramspace(p=1, M=2, d=2, params=theta_122_int2, same_means=list(1, 2)))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222c_int, constraints=C_222, same_means=list(1:2)))
   expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222csLAR_int, constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222), same_means=list(1:2)))
+  expect_true(in_paramspace(p=2, M=2, d=2, params=theta_222tcsLAR_int, model="StMVAR", constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222), same_means=list(1:2)))
   expect_false(in_paramspace(p=3, M=3, d=2, params=theta_332c_int, constraints=C_332, same_means=list(1, 2:3)))
+  expect_false(in_paramspace(p=3, M=c(2, 1), d=2, params=theta_332gsc_int, model="G-StMVAR", constraints=C_332, same_means=list(1, 2:3)))
   expect_false(in_paramspace(p=1, M=2, d=3, params=theta_123_int, same_means=list(1:2)))
+  expect_false(in_paramspace(p=1, M=c(1, 1), d=3, params=theta_123gs_int, model="G-StMVAR", same_means=list(1:2)))
   expect_true(in_paramspace(p=1, M=2, d=3, params=theta_123c_int, constraints=C_123, same_means=list(1:2)))
+
+  # Check df
+  expect_true(in_paramspace(p=1, M=2, d=2, params=theta_122_ok, model="GMVAR"))
+  expect_false(in_paramspace(p=1, M=2, d=2, params=theta_122t_df, model="StMVAR"))
+  expect_false(in_paramspace(p=1, M=c(1, 1), d=2, params=theta_122gs_df, model="G-StMVAR"))
 })
 
 test_that("check_parameters works correctly", {
-
   expect_error(check_parameters(p=1, M=1, d=2, params=theta_112))
+  expect_error(check_parameters(p=1, M=1, d=2, params=theta_112t, model="StMVAR"))
   expect_error(check_parameters(p=2, M=3, d=2, params=theta_222))
   expect_error(check_parameters(p=3, M=3, d=2, params=theta_332))
+  expect_error(check_parameters(p=3, M=c(1, 2), d=2, params=theta_332gs, model="G-StMVAR"))
   expect_error(check_parameters(p=1, M=2, d=3, params=theta_123))
+  expect_error(check_parameters(p=1, M=2, d=3, params=theta_123t, model="StMVAR"))
   expect_error(check_parameters(p=2, M=1, d=3, params=theta_213))
+  expect_error(check_parameters(p=2, M=1, d=3, params=theta_213t, model="StMVAR"))
 
   expect_error(check_parameters(p=1, M=1, d=2, params=theta_112c, constraints=C_112))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222c, constraints=t(C_222)))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222_c2, constraints=as.vector(C_222_2)))
 
-  # SGMVAR
+  # Structural
   expect_error(check_parameters(p=1, M=1, d=2, params=theta_112s, structural_pars=list(W=W_112)))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222s, structural_pars=list(xW=W_222)))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222s, structural_pars=list(W_222)))
+  expect_error(check_parameters(p=2, M=2, d=2, params=theta_222ts, model="StMVAR", structural_pars=list(W_222)))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222s, structural_pars=list(W=W_123)))
   expect_error(check_parameters(p=2, M=2, d=2, params=c(theta_222s, 1), structural_pars=list(W=W_222)))
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222s, structural_pars=list(W=W_222, C_lambda=matrix(1))))
@@ -435,16 +555,23 @@ test_that("check_parameters works correctly", {
   expect_error(check_parameters(p=2, M=2, d=2, params=theta_222c_int, constraints=C_222, same_means=list(1:2), parametrization="intercept"))
   expect_error(check_parameters(p=1, M=2, d=3, params=theta_123c_int, constraints=C_222, same_means=list(1:2), parametrization="mean"))
   expect_error(check_parameters(p=1, M=2, d=3, params=theta_123csL_int, same_means=list(1:2), structural_pars=list(W=W_123, C_lambda=C_lambda_123), parametrization="mean"))
+
+  # Check df
+  expect_error(check_parameters(p=1, M=2, d=2, params=theta_122t_df, model="StMVAR"))
+  expect_error(check_parameters(p=1, M=c(1, 1), d=2, params=theta_122gs_df, model="G-StMVAR"))
+
 })
 
 
 test_that("check_constraints works correctly", {
   expect_error(check_constraints(p=2, M=2, d=2, same_means=1:2))
+  expect_error(check_constraints(p=2, M=c(1, 1), d=2, same_means=1:2))
   expect_error(check_constraints(p=1, M=1, d=2, same_means=1:2))
   expect_error(check_constraints(p=2, M=2, d=2, same_means=list(1)))
   expect_error(check_constraints(p=2, M=2, d=2, same_means=list(1, "2")))
   expect_error(check_constraints(p=2, M=3, d=2, same_means=list(2:3)))
   expect_error(check_constraints(p=3, M=2, d=2, same_means=list(1, 1:2)))
+  expect_error(check_constraints(p=3, M=c(1, 1), d=2, same_means=list(1, 1:2)))
   expect_error(check_constraints(p=1, M=1, d=2, same_means=list()))
   expect_error(check_constraints(p=1, M=1, d=2, same_means=list(1:2)))
 
@@ -455,6 +582,7 @@ test_that("check_constraints works correctly", {
   expect_error(check_constraints(p=2, M=2, d=2, constraints=C_222[-1,]))
   expect_error(check_constraints(p=2, M=2, d=2, constraints=C_222_2[-16,]))
   expect_error(check_constraints(p=3, M=3, d=2, constraints=t(C_332)))
+  expect_error(check_constraints(p=3, M=c(2, 1), d=2, constraints=t(C_332)))
   expect_error(check_constraints(p=1, M=2, d=3, constraints=cbind(C_123[,1:8], C_123[,8])))
   expect_error(check_constraints(p=2, M=1, d=3, constraints=as.vector(C_213)))
 
@@ -470,6 +598,7 @@ test_that("check_constraints works correctly", {
   expect_error(check_constraints(p=1, M=1, d=2, structural_pars=list(W=t(W_bad))))
   C_lambda_bad <- matrix(c(1, -0.001), nrow=2)
   expect_error(check_constraints(p=1, M=2, d=2, structural_pars=list(W=W_122, C_lambda=C_lambda_bad)))
+  expect_error(check_constraints(p=1, M=c(1, 1), d=2, structural_pars=list(W=W_122, C_lambda=C_lambda_bad)))
 })
 
 
@@ -501,6 +630,13 @@ test_that("check_pMd works correctly", {
   expect_error(check_pMd(p=1, M=-1, d=2))
   expect_error(check_pMd(p=1.1, M=1, d=2))
   expect_error(check_pMd(p=1, M=1, d=2.2))
+
+  expect_error(check_pMd(p=1, M=1, d=2, model="G-StMVAR"))
+  expect_error(check_pMd(p=2, M=c(1, 1), d=2, model="StMVAR"))
+  expect_error(check_pMd(p=2, M=c(1, -1), d=3, model="G-StMVAR"))
+  expect_error(check_pMd(p=-1, M=1, d=2, model="StMVAR"))
+  expect_error(check_pMd(p=1, M=-1, d=2, model="StMVAR"))
+
 })
 
 test_that("all_pos_ints works correctly", {
@@ -532,60 +668,80 @@ theta_212c <- c(phi10_212, vec(A11_212), vec(A12_212), vech(Omega1_212))
 theta_212csWAR <- c(phi10_212, vec(A11_212), vec(A11_212), Wvec(W_212)) # SGMVAR W and AR
 
 test_that("n_params works correctly", {
-  # same_means
-  expect_equal(n_params(p=1, M=1, d=2, same_means=list(1)), length(theta_112_int))
-  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1:2)), length(theta_122_int))
-  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1, 2)), length(theta_122_int2))
-  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1:2), constraints=C_122, structural_pars=list(W=W_122, C_lambda=C_lambda_122)), length(theta_122csL_int))
-  expect_equal(n_params(p=2, M=2, d=2, same_means=list(1:2), constraints=C_222), length(theta_222c_int))
-  expect_equal(n_params(p=2, M=2, d=2, same_means=list(1:2), constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222)), length(theta_222csLAR_int))
-  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(1, 2:3)), length(theta_332c_int))
-  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(2, c(1, 3))), length(theta_332c_int2))
-  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(1:3)), length(theta_332c_int3))
-  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332), same_means=list(c(1, 3), 2)), length(theta_332csWAR_int))
-  expect_equal(n_params(p=3, M=3, d=2, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(1:2, 3)), length(theta_332csWL_int))
-  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(2:3, 1)), length(theta_332csWLAR_int))
-  expect_equal(n_params(p=1, M=2, d=3, same_means=list(1:2)), length(theta_123_int))
-  expect_equal(n_params(p=1, M=2, d=3, constraints=C_123, same_means=list(1:2)), length(theta_123c_int))
-  expect_equal(n_params(p=1, M=2, d=3, structural_pars=list(W=W_123, C_lambda=C_lambda_123), same_means=list(1:2)), length(theta_123csL_int))
-  expect_equal(n_params(p=1, M=2, d=3, constraints=C_123, structural_pars=list(W=W_123, C_lambda=C_lambda_123), same_means=list(1:2)), length(theta_123csLAR_int))
-  expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, same_means=list(1)), length(theta_213c_int))
-  expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, structural_pars=list(W=W_213), same_means=list(1)), length(theta_213csWAR_int))
-
   expect_equal(n_params(p=1, M=1, d=2), length(theta_112))
+  expect_equal(n_params(p=1, M=1, d=2, model="StMVAR"), length(theta_112t))
   expect_equal(n_params(p=1, M=2, d=2), length(theta_122))
   expect_equal(n_params(p=2, M=2, d=2), length(theta_222))
+  expect_equal(n_params(p=2, M=c(1, 1), d=2, model="G-StMVAR"), length(theta_222gs))
   expect_equal(n_params(p=3, M=3, d=2), length(theta_332))
+  expect_equal(n_params(p=3, M=c(1, 2), d=2, model="G-StMVAR"), length(theta_332gs))
   expect_equal(n_params(p=1, M=2, d=3), length(theta_123))
+  expect_equal(n_params(p=1, M=2, d=3, model="StMVAR"), length(theta_123t))
   expect_equal(n_params(p=2, M=1, d=3), length(theta_213))
+  expect_equal(n_params(p=2, M=1, d=3, model="StMVAR"), length(theta_213t))
   expect_equal(n_params(p=2, M=1, d=2), length(theta_212))
 
   expect_equal(n_params(p=1, M=1, d=2, constraints=C_112), length(theta_112c))
+  expect_equal(n_params(p=1, M=1, d=2, constraints=C_112, model="StMVAR"), length(theta_112tc))
   expect_equal(n_params(p=1, M=2, d=2, constraints=C_122), length(theta_122c))
   expect_equal(n_params(p=2, M=2, d=2, constraints=C_222), length(theta_222c))
   expect_equal(n_params(p=3, M=3, d=2, constraints=C_332), length(theta_332c))
+  expect_equal(n_params(p=3, M=c(2, 1), d=2, constraints=C_332, model="G-StMVAR"), length(theta_332gsc))
   expect_equal(n_params(p=1, M=2, d=3, constraints=C_123), length(theta_123c))
   expect_equal(n_params(p=2, M=1, d=3, constraints=C_213), length(theta_213c))
   expect_equal(n_params(p=2, M=1, d=2, constraints=C_212), length(theta_212c))
 
-  # SGMVAR
+  # Structural
   expect_equal(n_params(p=1, M=1, d=2, structural_pars=list(W=W_112)), length(theta_112sWC))
+  expect_equal(n_params(p=1, M=1, d=2, structural_pars=list(W=W_112), model="StMVAR"), length(theta_112tsWC))
   expect_equal(n_params(p=1, M=2, d=2, structural_pars=list(W=W_122)), length(theta_122s))
   expect_equal(n_params(p=2, M=2, d=2, structural_pars=list(W=W_222, C_lambda=C_lambda_222)), length(theta_222csL))
   expect_equal(n_params(p=3, M=3, d=2, structural_pars=list(W=W_332)), length(theta_332sWC))
+  expect_equal(n_params(p=3, M=c(2, 1), d=2, structural_pars=list(W=W_332), model="G-StMVAR"), length(theta_332gssWC))
   expect_equal(n_params(p=1, M=2, d=3, structural_pars=list(W=W_123)), length(theta_123s))
+  expect_equal(n_params(p=1, M=c(1, 1), d=3, structural_pars=list(W=W_123), model="G-StMVAR"), length(theta_123gss))
   expect_equal(n_params(p=1, M=2, d=3, structural_pars=list(W=W_123, C_lambda=C_lambda_123)), length(theta_123csL))
   expect_equal(n_params(p=2, M=1, d=3, structural_pars=list(W=W_213)), length(theta_213sWC))
+  expect_equal(n_params(p=2, M=1, d=3, structural_pars=list(W=W_213), model="StMVAR"), length(theta_213tsWC))
   expect_equal(n_params(p=2, M=1, d=2, structural_pars=list(W=W_212)), length(theta_212sWC))
 
   expect_equal(n_params(p=1, M=1, d=2, constraints=C_112, structural_pars=list(W=W_112)), length(theta_112csWAR))
+  expect_equal(n_params(p=1, M=1, d=2, constraints=C_112, structural_pars=list(W=W_112), model="StMVAR"), length(theta_112tcsWAR))
   expect_equal(n_params(p=1, M=2, d=2, constraints=C_122, structural_pars=list(W=W_122, C_lambda=C_lambda_122)), length(theta_122csLAR))
   expect_equal(n_params(p=2, M=2, d=2, constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222)), length(theta_222csLAR))
+  expect_equal(n_params(p=2, M=2, d=2, constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222), model="StMVAR"), length(theta_222tcsLAR))
   expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332, C_lambda=C_lambda_332)), length(theta_332csWLAR))
+  expect_equal(n_params(p=3, M=c(1, 2), d=2, constraints=C_332, structural_pars=list(W=W_332, C_lambda=C_lambda_332), model="G-StMVAR"), length(theta_332gscsWLAR))
   expect_equal(n_params(p=1, M=2, d=3, constraints=C_123, structural_pars=list(W=W_123, C_lambda=C_lambda_123)), length(theta_123csLAR))
   expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, structural_pars=list(W=W_213)), length(theta_213csWAR))
+  expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, structural_pars=list(W=W_213), model="StMVAR"), length(theta_213tcsWAR))
   expect_equal(n_params(p=2, M=1, d=2, constraints=C_212, structural_pars=list(W=W_212)), length(theta_212csWAR))
 
+  # same_means
+  expect_equal(n_params(p=1, M=1, d=2, same_means=list(1)), length(theta_112_int))
+  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1:2)), length(theta_122_int))
+  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1:2), model="StMVAR"), length(theta_122t_int))
+  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1, 2)), length(theta_122_int2))
+  expect_equal(n_params(p=1, M=2, d=2, same_means=list(1:2), constraints=C_122, structural_pars=list(W=W_122, C_lambda=C_lambda_122)), length(theta_122csL_int))
+  expect_equal(n_params(p=2, M=2, d=2, same_means=list(1:2), constraints=C_222), length(theta_222c_int))
+  expect_equal(n_params(p=2, M=2, d=2, same_means=list(1:2), constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222)), length(theta_222csLAR_int))
+  expect_equal(n_params(p=2, M=2, d=2, model="StMVAR", same_means=list(1:2), constraints=C_222, structural_pars=list(W=W_222, C_lambda=C_lambda_222)),
+               length(theta_222tcsLAR_int))
+  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(1, 2:3)), length(theta_332c_int))
+  expect_equal(n_params(p=3, M=c(2, 1), d=2, model="G-StMVAR", constraints=C_332, same_means=list(1, 2:3)), length(theta_332gsc_int))
+  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(2, c(1, 3))), length(theta_332c_int2))
+  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, same_means=list(1:3)), length(theta_332c_int3))
+  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332), same_means=list(c(1, 3), 2)), length(theta_332csWAR_int))
+  expect_equal(n_params(p=3, M=3, d=2, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(1:2, 3)), length(theta_332csWL_int))
+  expect_equal(n_params(p=3, M=c(1, 2), d=2, model="G-StMVAR", structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(1:2, 3)), length(theta_332gscsWL_int))
+  expect_equal(n_params(p=3, M=3, d=2, constraints=C_332, structural_pars=list(W=W_332, C_lambda=C_lambda_332), same_means=list(2:3, 1)), length(theta_332csWLAR_int))
+  expect_equal(n_params(p=1, M=2, d=3, same_means=list(1:2)), length(theta_123_int))
+  expect_equal(n_params(p=1, M=2, d=3, constraints=C_123, same_means=list(1:2)), length(theta_123c_int))
+  expect_equal(n_params(p=1, M=2, d=3, structural_pars=list(W=W_123, C_lambda=C_lambda_123), same_means=list(1:2)), length(theta_123csL_int))
+  expect_equal(n_params(p=1, M=2, d=3, model="StMVAR", structural_pars=list(W=W_123, C_lambda=C_lambda_123), same_means=list(1:2)), length(theta_123tcsL_int))
+  expect_equal(n_params(p=1, M=2, d=3, constraints=C_123, structural_pars=list(W=W_123, C_lambda=C_lambda_123), same_means=list(1:2)), length(theta_123csLAR_int))
+  expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, same_means=list(1)), length(theta_213c_int))
+  expect_equal(n_params(p=2, M=1, d=3, constraints=C_213, structural_pars=list(W=W_213), same_means=list(1)), length(theta_213csWAR_int))
 })
 
 params44 <- c(47.2024, 124.5639, 1.0384, -2.4929, -0.1152, 1.5416, -0.7323, 4.0717, 0.0956, -0.5456, -0.0806, -5.9224,
@@ -595,7 +751,7 @@ params44 <- c(47.2024, 124.5639, 1.0384, -2.4929, -0.1152, 1.5416, -0.7323, 4.07
               -0.5555, 0.4148, -0.0845, -0.0039, 0.5894, -0.3778, 0.0617, 0.0695, -0.3935, 5.4352, 3.7143, 7.4973, -16.3047,
               48.9572, 0.8365, -0.0827, 0.1168, 1.0636, 0.3199, 1.9638, 0.004, -0.3164, -0.5723, -0.2664, 0.0047, -0.1401,
               0.2292, -1.2575, -0.0081, 0.0308, 0.5333, 0.8176, 4.0922, 0.6454, 0.1588, 0.1076)
-mod44 <- GMVAR(p=4, M=4, d=2, params=params44)
+mod44 <- GSMVAR(p=4, M=4, d=2, params=params44)
 
 params44_2 <- c(-13.9054, 20.1997, 0.6993, 0.27, 0.3735, 0.9154, -0.1393, -0.2023, -0.2395, -0.3351, 0.3318, -0.1009,
                 -0.1445, 0.3762, 0.06, -0.0415, 0.1408, -0.102, 25.866, 5.9786, 13.54, -16.6969, -9.5959, 0.4457,
@@ -605,13 +761,23 @@ params44_2 <- c(-13.9054, 20.1997, 0.6993, 0.27, 0.3735, 0.9154, -0.1393, -0.202
                 0.231, 3.6336, -1.8297, 18.0466, 13.7945, 80.9443, 1.5888, 0.239, 0.0985, 0.7906, -0.5625, 0.1164,
                 -0.2218, -0.2726, -0.0398, -0.3315, 0.3602, -0.208, 0.0263, 0.2054, -0.3508, 0.0475, 3.8855,
                 -3.2228, 20.6817, 0.5301, 0.2762, 0.1756)
-mod44_2 <- GMVAR(p=4, M=4, d=2, params=params44_2)
+mod44_2 <- GSMVAR(p=4, M=4, d=2, params=params44_2)
+
+mod44t_2 <- suppressWarnings(GSMVAR(p=4, M=4, d=2, params=c(params44_2, 10, 20, 30, 500), model="StMVAR"))
+mod44gs_2 <- suppressWarnings(GSMVAR(p=4, M=c(2, 2), d=2, params=c(params44_2, 30, 400), model="G-StMVAR"))
 
 params112 <- c(1.03, 2.36, 0.85, 0.06, 0.04, 0.94, 0.93, -0.913, 0.90)
-mod112 <- GMVAR(p=1, M=1, d=2, params=params112)
+mod112 <- GSMVAR(p=1, M=1, d=2, params=params112)
 
 test_that("warn_eigens works correctly", {
   expect_warning(warn_eigens(mod44, tol=0.002))
   expect_warning(warn_eigens(mod44_2, tol=0.002))
+  expect_warning(warn_eigens(mod44t_2, tol=0.002))
+  expect_warning(warn_eigens(mod44gs_2, tol=0.002))
   expect_warning(warn_eigens(mod112, tol=0.002))
 })
+
+test_that("warn_eigens works correctly", {
+  expect_warning(warn_eigens(mod44t_2))
+  expect_warning(warn_eigens(mod44gs_2))
+})
